@@ -1,4 +1,6 @@
-from typing import Any, Dict, List
+import operator
+from collections import defaultdict
+from typing import Any, Dict, Iterable, List
 
 import numpy as np
 import pandas
@@ -16,7 +18,7 @@ def df_to_json_table(df, *args, **kwargs):
 
     Parameters
     ----------
-    df : pd.Dataframe
+    df : pandas.Dataframe
         Input dataframe
     *args: passed on to df.to_dict
     **kwargs: passed on to df.to_dict
@@ -37,10 +39,10 @@ def df_to_json_table(df, *args, **kwargs):
 def df_to_dict(df):
     """Create a dictionary from pandas dataframes
     Types are converted to native Python types using pandas' built-in
-    mapping (see pd.DataFrame.to_dict()).
+    mapping (see pandas.DataFrame.to_dict()).
     Parameters
     ----------
-    df : pd.Dataframe
+    df : pandas.Dataframe
         Input dataframe
     Returns
     -------
@@ -59,10 +61,10 @@ def create_description(series, n_options=100) -> Dict:
 
     Parameters
     ----------
-    series : pd.Series
+    series : pandas.Series
         Pandas Series to be described
     n_options : int, optional
-        Number of options for dropdown menus, by default 100
+        Number of options for dropandown menus, by default 100
 
     Returns
     -------
@@ -118,10 +120,73 @@ def vega_sanitize(var, datatype=None):
     return var
 
 
+def format_time(X, var):
+    """Format datetime for vega plots"""
+
+    timedelta = X[var].max() - X[var].min()
+    if timedelta.days > 1:
+        format = "%x"
+    else:
+        format = "%X"
+    return format
+
+
 def input_schema_to_pandas(value: Any, names: List[str]) -> pandas.DataFrame:
-    print(names, value)
     if isinstance(value, pandas.DataFrame):
         value.columns = names
         return value
     else:
         return pandas.DataFrame(value, columns=names)
+
+
+def batched_non_object_col_mode(
+    x: Iterable[pandas.DataFrame],
+    y: Iterable[pandas.Series],
+    first: pandas.DataFrame,
+    col: str,
+):
+    non_object_modes = defaultdict(int)
+    values, counts = np.unique(first[col].dropna().values, return_counts=True)
+
+    for v, c in zip(values, counts):
+        non_object_modes[v] += c
+    for batch, label in zip(x, y):
+        batch = drop_nas_from_batch(batch, label)
+        values, counts = np.unique(batch[col].dropna(), return_counts=True)
+        for v, c in zip(values, counts):
+            non_object_modes[v] += c
+
+    items = non_object_modes.items()
+
+    if items:
+        return max(non_object_modes.items(), key=operator.itemgetter(1))[0]
+    else:
+        return None
+
+
+def batched_numeric_col_mean(
+    x: Iterable[pandas.DataFrame],
+    y: Iterable[pandas.Series],
+    first: pandas.DataFrame,
+    col: str,
+):
+    running_sum, running_length = first[col].dropna().sum(), len(first[col].dropna())
+    for batch, label in zip(x, y):
+        batch = drop_nas_from_batch(batch, label)
+        clean = batch[col].dropna()
+        running_sum += clean.sum()
+        running_length += len(clean)
+
+    if running_length:
+        return running_sum / running_length
+    else:
+        return float("NaN")
+
+
+def drop_nas_from_batch(x: pandas.DataFrame, y: pandas.Series) -> pandas.DataFrame:
+    not_missing = [i for i, v in enumerate(y) if not pandas.isna(v)]
+    n_missing = len(y) - len(not_missing)
+    if n_missing > 0:
+        return x.iloc[not_missing]
+    else:
+        return x
